unit ssimgfrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, dsgnIntf;

type
  TfrmColorSelector = class(TForm)
    lblCopyright: TLabel;
    sbHor: TScrollBar;
    sbVer: TScrollBar;
    panImage: TPanel;
    imgShow: TImage;
    panButtons: TPanel;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    btnReset: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    lblRed: TLabel;
    lblGreen: TLabel;
    lblBlue: TLabel;
    shpToStore: TShape;
    shpColor: TShape;
    procedure btnResetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sbHorChange(Sender: TObject);
    procedure sbVerChange(Sender: TObject);
    procedure ReDrawPicture;
    procedure imgShowMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    function min(a, b: longint): longint;
    function max(a, b: longint): longint;
    procedure imgShowMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure imgShowMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure imgShowDblClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    OrigPict, WorkPict : TBitmap;
    VertS, HorS : longint;
    Select: boolean;
    sx, ex, sy, ey : longint;
  end;

type
  TTransColorProperty = class(TIntegerProperty)
  public
    dialog : TfrmColorSelector;
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

var
  frmColorSelector: TfrmColorSelector;

implementation

{$R *.DFM}

function TTransColorProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog];
end;

procedure TTransColorProperty.Edit;
begin
  if TImage(Self.GetComponent(0)).Picture.Bitmap.Empty then
  begin
    MessageDlg('The component is does not content any bitmap.', mtError, [mbOk], 0);
    exit;
  end;
  dialog := TfrmColorSelector.Create(Application);
  try
    dialog.OrigPict.Assign(TImage(Self.GetComponent(0)).Picture.Bitmap);
    dialog.WorkPict.Assign(dialog.OrigPict);
    dialog.btnResetClick(Self);
    if dialog.ShowModal = mrOk then begin
      SetOrdValue(dialog.shpToStore.Brush.Color);
      TImage(Self.GetComponent(0)).Picture.Bitmap.TransparentMode := tmFixed;
      TImage(Self.GetComponent(0)).Picture.Bitmap.TransparentColor :=
        dialog.shpToStore.Brush.Color;
      TImage(Self.GetComponent(0)).Invalidate;
    end;
  finally
    dialog.Release;
  end;
end;

function  TfrmColorSelector.min(a, b: longint): longint;
begin
  if a < b then Result := a else Result := b;
end;

function  TfrmColorSelector.max(a, b: longint): longint;
begin
  if a > b then Result := a else Result := b;
end;

procedure TfrmColorSelector.btnResetClick(Sender: TObject);
begin
  // Reset image to the original
  sbHor.Enabled := True; sbVer.Enabled := True;
  if (imgShow.Width < WorkPict.Width) then
    begin
      sbHor.Min := 0; sbHor.Max := max(imgShow.Width, origPict.Width) -
        min(imgShow.Width, origPict.Width)
    end
  else
    begin
      sbHor.Min := 0; sbHor.Max := 0;
    end;

  if imgShow.Height < WorkPict.Height then
    begin
      sbVer.Min := 0; sbVer.Max := max(imgShow.Height, origPict.Height) -
        min(imgShow.Height, origPict.Height)
    end
  else
    begin
      sbVer.Min := 0; sbVer.Max := 0;
    end;
  sbHor.Position := 0; sbVer.Position := 0;
  imgShow.Picture := nil;
  imgShow.Canvas.Draw(0, 0, OrigPict);
end;

procedure TfrmColorSelector.FormCreate(Sender: TObject);
begin
  OrigPict := TBitmap.Create; WorkPict := TBitmap.Create;
end;

procedure TfrmColorSelector.sbHorChange(Sender: TObject);
begin
  // Adjust visible image to be consist with the scrollbars
  HorS := sbHor.Position;
  ReDrawPicture;
end;

procedure TfrmColorSelector.sbVerChange(Sender: TObject);
begin
  // Adjust visible image to be consist with the scrollbars
  VertS := sbVer.Position;
  ReDrawPicture;
end;

procedure TfrmColorSelector.ReDrawPicture;
begin
  imgShow.Picture := nil;
  imgShow.Canvas.Draw(0-HorS, 0-VertS, WorkPict);
end;

procedure TfrmColorSelector.imgShowMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  tmpclr: string;
begin
  if not Select then
  begin
    tmpclr := inttohex(imgShow.Canvas.Pixels[x, y], 6);
    shpColor.Brush.Color := imgShow.Canvas.Pixels[x, y];
    lblRed.Caption := copy(tmpclr, 1, 2);
    lblGreen.Caption := copy(tmpclr, 3, 2);
    lblBlue.Caption := copy(tmpclr, 5, 2);
  end
  else
    begin
      TImage(Sender).Canvas.DrawFocusRect(Rect(min(sx, ex),min(sy, ey),max(sx, ex), max(sy,ey)));
      ex := x; ey := y;
      TImage(Sender).Canvas.DrawFocusRect(Rect(min(sx, ex),min(sy, ey),max(sx, ex), max(sy,ey)));
  end;
end;

procedure TfrmColorSelector.imgShowMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  temp: TBitmap;
  myrect, myorect:    TRect;
begin
  myrect := Rect(min(sx, ex),min(sy, ey),max(sx, ex), max(sy,ey));
  TImage(Sender).Canvas.DrawFocusRect(myrect);
  Select := False;
  if (((myrect.Right - myrect.Left) = 0) or
      ((myrect.Bottom - myrect.Top) = 0)) then
  begin
    exit;
  end;
  sbHor.Enabled := False; sbVer.Enabled := False;
  temp := TBitmap.Create;
  temp.Width  := myRect.Right - myRect.Left; temp.Height := myRect.Bottom - myRect.Top;
  myorect.Left := 0; myorect.Top := 0; myorect.Right := myrect.Right - myrect.Left;
  myorect.Bottom := myrect.Bottom - myrect.Top;
  temp.Canvas.CopyRect(myorect, imgShow.Canvas, myRect);
  imgShow.Picture := nil;
  imgShow.Canvas.StretchDraw(imgShow.ClientRect, temp);
  temp.Free;
end;

procedure TfrmColorSelector.imgShowMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Select := True;
  sx := x; sy := y; ex := x; ey := y;
  TImage(Sender).Canvas.DrawFocusRect(Rect(sx,sy,ex,ey));
end;

procedure TfrmColorSelector.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  OrigPict.Free; WorkPict.Free;
end;

procedure TfrmColorSelector.imgShowDblClick(Sender: TObject);
begin
  shpToStore.Brush.Color := shpColor.Brush.Color;
end;

end.
